﻿import Random;

public class CLS_ScatterPoints
{
	
	
	/// <summary>
	/// This function scatters points across the surface of a gameobjects mesh, with an option to set the seed.
	/// </summary>
	/// <returns>This function returns an array of Vector points.</returns>
	public static function Scatter(obj:GameObject, numberOfPoints:int, seed:int)
	{
		var oMesh:Mesh =  obj.GetComponent.<MeshFilter>().sharedMesh;
		
		// Apply world space transformations
		var verts:Vector3[] =  oMesh.vertices;
		for(var i:int=0;i<verts.Length;++i)
		{
			verts[i] = obj.transform.TransformPoint(verts[i]);
		}
		
		
		
		
		return Scatter(verts, oMesh.triangles, numberOfPoints, seed);
	}
	
	
	/// <summary>
	/// This function scatters points across the surface of a gameobjects mesh.
	/// </summary>
	/// <returns>This function returns an array of Vector points.</returns>
	public static function Scatter(obj:GameObject, numberOfPoints:int)
	{
		var oMesh:Mesh =  obj.GetComponent.<MeshFilter>().sharedMesh;
		
		
		// Apply world space transformations
		var verts:Vector3[] =  oMesh.vertices;
		for(var i:int=0;i<verts.Length;++i)
		{
			verts[i] = obj.transform.TransformPoint(verts[i]);
		}
		
		
		return Scatter(verts, oMesh.triangles, numberOfPoints);
	}
	
	
	
	/// <summary>
	/// This function scatters points across the surface of a gameobjects mesh, with an option to set the seed.
	/// </summary>
	/// <returns>This function returns an array of Vector points.</returns>
	public static function Scatter(vertices : Vector3[], triangles:int[], numberOfPoints:int, seed:int)
	{
		var pseed:int = Random.seed;
		Random.seed = seed;
		var ret:Vector3[] = Scatter(vertices, triangles, numberOfPoints);
		Random.seed = pseed;
		return ret;
	
	}
	
	/// <summary>
	/// This function scatters points across the surface of a gameobjects mesh.
	/// </summary>
	/// <returns>This function returns an array of Vector points.</returns>
	public static function Scatter(vertices : Vector3[], triangles:int[], numberOfPoints:int)
	{
		var ret:Vector3[] = new Vector3[numberOfPoints];
		var nTri:int = triangles.length/3; // Number of different triangles
		
		for(var i:int=0;i<numberOfPoints;++i)
		{
			var cTri:int = Mathf.FloorToInt(Random.Range(0,nTri))*3; //Multiply by 3 to get a triangles index
			
			
			var A:Vector3 = vertices[triangles[cTri]];
			var B:Vector3 = vertices[triangles[cTri+1]];
			var C:Vector3 = vertices[triangles[cTri+2]];
			
			// Interperolate inbetween all sides of the triangle, to get somewhere in between
			ret[i] =  Vector3.Lerp(Vector3.Lerp(A,B,Random.value),C,Random.value);
			
		}
		return ret;
	}
	
}

